/*
********************************************************************

    this program calculates the data complexity of a project with
    the file generated by rule file dcomp.cc as input. the code in
    this program is strictly in ANSI C and should have no problem
    with most of C compilers. if the program is compiled with
    compiler option -DPRINT, the result for each defined function
    will be displayed, otherwise only the result for whole project
    is displayed.

********************************************************************
*/
#include <stdio.h>
#include <stdlib.h>

#define MAX_LINE_LEN 1024
#define FCN_1ST_CHARS 53
#define STRING_END '\0'

#ifndef NULL
#define NULL 0
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

typedef struct FcnInfoRec *FIRPtr;

struct FcnInfoRec    /* record of function infomation */
{
    char*  fcnName;
    char*  modName;
    int    fanIn;
    int    fanOut;
    int    ioVars;
    int    isGlobal;
    FIRPtr next;
};

typedef struct CalleeInfoRec *CIRPtr;

struct CalleeInfoRec    /* record of called function information */
{
    char*  calleeName;
    char*  modName;
    FIRPtr callerPtr;
    CIRPtr next;
};

void main( int argc, char** argv )
{
    FILE  *fd;
    char  line[ MAX_LINE_LEN ];
    char*  fcnName;
    char*  modName;
    char*  calleeName;
    CIRPtr calleeRecPtr,
           calleeList[FCN_1ST_CHARS],
           prevCallee;
    FIRPtr prev,
           curr,
           fcnRecPtr,
           global,
           local,
           fcnList[53];
    int    len;
    int    fcnCount;
    float  complex;
    float  complex1;
    int    comp;
    int    index;

    if ( argc != 2 )
    {
        printf( "%s should be run with a input file\n", argv[0] );
        exit( -1 );
    }

    fd = fopen( argv[1], "r" );
    if ( fd == NULL )
    {
        printf( "fail to open file %s\n", argv[1] );
        exit( -1 );
    }

    for ( index = 0; index < FCN_1ST_CHARS ; index ++ )
    {
         fcnList[index] = NULL;
         calleeList[index] = NULL; 
    }
    while ( fgets( line, MAX_LINE_LEN, fd ) )
    {
        line[ strlen( line ) - 1 ] = STRING_END;

        switch ( line[0] )
        {
            case '!' :
                if ( strncmp(  line + 1, "begin", 5 ) == 0  )
                {
                    len = strlen( line + 7 ) + 1;
                    modName = (char*)malloc( len );
                    strcpy( modName, line + 7 ); 
                }
                else
                {
                    free( modName );
                    modName = NULL;
                }
                break;

            case '$' :
                if ( strncmp( line + 2, "begin", 5 ) == 0 )
                {
                    len = strlen( line + 8 ) + 1;
                    fcnName = (char*)malloc( len );
                    strcpy( fcnName, line + 8 ); 
                    fcnRecPtr = (FIRPtr)malloc( sizeof( struct FcnInfoRec ) );
                    fcnRecPtr->modName = (char*)malloc( strlen( modName ) + 1 );
                    strcpy( fcnRecPtr->modName, modName );
                    fcnRecPtr->fcnName = (char*)malloc( strlen( fcnName ) + 1 );
                    strcpy( fcnRecPtr->fcnName, fcnName );
                    fcnRecPtr->isGlobal = ( line[1] == 'g' ) ? TRUE : FALSE;
                    fcnRecPtr->ioVars = 0;
                    fcnRecPtr->fanIn = 0;
                    fcnRecPtr->fanOut = 0;
                    fcnRecPtr->next = NULL;
                    if ( fcnName[0] == '_' )
                    {
                        index = 0;
                    }
                    else
                    {
                        if ( fcnName[0] >= 'A' && fcnName[0] <= 'Z' )
                        {
                            index = fcnName[0] - 'A' + 1;
                        }
                        else
                        {
                             index = fcnName[0] - 'a' + 27;
                        }
                    }
                    if ( fcnList[index] == NULL )
                    {
                        fcnList[index] = fcnRecPtr;
                    }
                    else
                    {
                        prev = NULL;
                        curr = fcnList[index];
                        while ( curr != NULL )
                        {
                            if ( strcmp( fcnName, curr->fcnName ) >= 0 )
                            {
                                prev = curr;
                                curr = curr->next;
                            }
                            else
                            {
                                if ( prev == NULL )
                                {
                                    fcnList[index] = fcnRecPtr;
                                    fcnRecPtr->next = curr;
                                }
                                else
                                {
                                    prev->next = fcnRecPtr;
                                    fcnRecPtr->next = curr; 
                                }
                                break;
                            }
                        }
                        if ( curr == NULL )
                        {
                            prev->next = fcnRecPtr;
                        }                            
                    }
                }
                else
                {
                    free( fcnName );
                    fcnName = NULL;
                }
                break;

            case '#' :
                fcnRecPtr->ioVars = atoi( line + 1 );
                break;

            default :
                len = strlen( line ) + 1;
                calleeName = (char*)malloc( len );
                strcpy( calleeName, line );
                calleeRecPtr = (CIRPtr)malloc( sizeof( struct CalleeInfoRec ) );
                calleeRecPtr->calleeName = calleeName;
                calleeRecPtr->callerPtr = fcnRecPtr;
                calleeRecPtr->modName = (char*)malloc( strlen( modName ) + 1 );
                strcpy( calleeRecPtr->modName, modName );
                if ( calleeName[0] == '_' )
                {
                    index = 0;
                }
                else
                {
                    if ( calleeName[0] >= 'A' && calleeName[0] <= 'Z' )
                    {
                        index = calleeName[0] - 'A' + 1;
                    }
                    else
                    {
                        index = calleeName[0] - 'a' + 27; 
                    }
                }

                if ( calleeList[index] == NULL )
                {
                    calleeRecPtr->next = NULL;
                    calleeList[index] = calleeRecPtr;
                }
                else
                {
                    calleeRecPtr->next = calleeList[index];
                    calleeList[index] = calleeRecPtr;
                }
                fcnRecPtr->fanOut++;
                break;
        }
    }

    for ( index = 0; index < FCN_1ST_CHARS; index++ )
    {
        prevCallee = NULL;
        calleeRecPtr = calleeList[index];
        while ( calleeRecPtr != NULL )
        {
            local = NULL;
            global = NULL;
            fcnRecPtr = fcnList[index];
            while ( fcnRecPtr != NULL )
            {
                comp = strcmp( fcnRecPtr->fcnName, calleeRecPtr->calleeName );
                if ( comp == 0 )
                {
                    if ( fcnRecPtr->isGlobal )
                    {
                        global = fcnRecPtr;
                    }
                    else
                    {
                        if ( strcmp( fcnRecPtr->modName, calleeRecPtr->modName ) == 0 )
                        {
                            local = fcnRecPtr;
                        }
                    }
                }
                else
                {
                    if ( comp > 0 )
                    {
                        break;
                    }
                }

                fcnRecPtr = fcnRecPtr->next;
            }

            if ( local != NULL || global != NULL )
            {
                if ( local != NULL )
                {
                    local->fanIn++;
                }
                else
                {
                    global->fanIn++;
                }
                if ( prevCallee == NULL )
                {
                    calleeList[index] = calleeRecPtr->next;
                    calleeRecPtr = calleeRecPtr->next;
                }
                else
                {
                    prevCallee->next = calleeRecPtr->next;
                    calleeRecPtr = calleeRecPtr->next;
                }
            }
            else
            {
                prevCallee = calleeRecPtr;
                calleeRecPtr = calleeRecPtr->next;
            }
        }
    }

    complex = 0.0;
    fcnCount = 0;
    for ( index = 0; index < FCN_1ST_CHARS; index++ )
    {
        fcnRecPtr = fcnList[index];
        while ( fcnRecPtr != NULL )
        {
            complex1 = fcnRecPtr->ioVars / ( fcnRecPtr->fanOut + 1.0 );
            complex += complex1;
#ifdef PRINT
            printf( "Function %s defined in %s\n", fcnRecPtr->fcnName, fcnRecPtr->modName );
            printf( "\tFan-In : %d\n", fcnRecPtr->fanIn );
            printf( "\tFan-Out : %d\n", fcnRecPtr->fanOut );
            printf( "\tIO-Vars : %d\n", fcnRecPtr->ioVars );
            printf( "\tData complexity : %f\n", complex1 );
#endif
            fcnCount++;
            fcnRecPtr = fcnRecPtr->next;
        }
    }

    complex /=  fcnCount;

    printf( "Project data complexity = %f\n", complex );
    fclose( fd );

    return;
}
