/*	conv.cc
	Copyright (c) 1994 by Abraxas Software. All rights reserved.
==============================================================================
	Purpose:	Detects implicit type conversions.
	Author:		Loren Cobb.
	Revision:	6 Aug 1994.
	Format:		Monospaced font with 4 spaces/tab.
==============================================================================

Abstract:

	This rule file prints a list of potentially dangerous implicit type
	conversions on stdout. Each type conversion is described, and the
	file and line number in which the conversion occurred is given.
	
	The specific implicit conversions detected are:
	
	1.  From a pointer type to a different pointer type.
	
	2.  From a signed integer to an unsigned integer.
	
	3.  From a longer integer to a shorter integer.

==============================================================================
*/

#include <check.cch>

int 	j, k, last_level, print_base, print_level,
		print_flags, print_operand;

if ( cnv_ptr_to_ptr )
	{
	printf( "\n# Implicit ptr->ptr conversion from " );
	print_operand = 1;
	printf( " to " );
	print_operand = 2;
	printf( "\n" );
	printf( "  File %s.c; Line %d\n", mod_name(), lin_number );
	}

if ( cnv_signed_to_any )
	{
	printf( "\n# Implicit signed->unsigned conversion from " );
	print_operand = 1;
	printf( " to " );
	print_operand = 2;
	printf( "\n" );
	printf( "  File %s.c; Line %d\n", mod_name(), lin_number );
	}

if ( cnv_truncate )
	{
	printf( "\n# Implicit trunction from " );
	print_operand = 1;
	printf( " to " );
	print_operand = 2;
	printf( "\n" );
	printf( "  File %s.c; Line %d\n", mod_name(), lin_number );
	}

if ( mod_end )
	printf( "\n" );

if ( print_operand )
	{
	printf( "(" );
	j = print_operand;
	last_level = op_levels( j );
	k = 0;
	while ( k <= last_level )
		{
		print_flags = op_level_flags( j, k );
		print_level = op_level( j, k++ );
		}
	print_base = op_base( j );
	printf( ")" );
	}

if ( print_flags )
	{
	if ( print_flags & CONST_FLAG )
		printf( "constant " );
	if ( print_flags & VOLATILE_FLAG )
		printf( "volatile " );
	if ( print_flags & NEAR_FLAG )
		printf( "near " );
	if ( print_flags & FAR_FLAG )
		printf( "far " );
	if ( print_flags & HUGE_FLAG )
		printf( "huge " );
	if ( print_flags & EXPORT_FLAG )
		printf( "export " );
	}

if ( print_level )
	{
	switch( print_level )
		{
	case ARRAY:
		printf( "array of " );
		break;
	case POINTER:
		printf( "ptr to " );
		break;
	case REFERENCE:
		printf( "ref to " );
		break;
	case FUNCTION:
		printf( "fcn returning " );
		break;
		}
	}

if ( print_base )
	{
	switch( print_base )
		{
	case VOID_TYPE:
		printf( "void" );
		break;
	case CHAR_TYPE:
		printf( "char" );
		break;
	case SHORT_TYPE:
		printf( "short" );
		break;
	case SHORT_TYPE:
		printf( "short" );
		break;
	case INT_TYPE:
		printf( "int" );
		break;
	case LONG_TYPE:
		printf( "long" );
		break;
	case UCHAR_TYPE:
		printf( "unsigned char" );
		break;
	case USHORT_TYPE:
		printf( "unsigned short" );
		break;
	case UINT_TYPE:
		printf( "unsigned int" );
		break;
	case ULONG_TYPE:
		printf( "unsigned long" );
		break;
	case FLOAT_TYPE:
		printf( "float" );
		break;
	case DOUBLE_TYPE:
		printf( "double" );
		break;
	case LONG_DOUBLE_TYPE:
		printf( "long double" );
		break;
	case ENUM_TYPE:
		printf( "enum" );
		printf( " %s", op_base_name(j) ); 
		break;
	case UNION_TYPE:
		printf( "union" );
		printf( " %s", op_base_name(j) ); 
		break;
	case STRUCT_TYPE:
		printf( "struct" );
		printf( " %s", op_base_name(j) ); 
		break;
	case CLASS_TYPE:
		printf( "class" );
		printf( " %s", op_base_name(j) ); 
		break;
	case DEFINED_TYPE:
		printf( "%s", op_base_name(j) ); 
		break;
	default:
		printf( "<unknown> = %d", print_base );
		break;
		}
	}
