//  Copyright (c) 1992-93 by Abraxas Software.

//  This is the start of a collection of CodeCheck rules
//  for elementary checking of C++ source code.


#include <check.cch>

if ( mod_begin )
	if ( (option('K') < 4) || (option('K') > 7) )
		{
		warn( 1000, "Use C++ for this file! (option -K4)." );
		}

//////////////////////////////////////////////////////////////////////////
//  1. Declare a virtual destructor in every base class that 
//     has a virtual function. (Stroustrup & Ellis, p. 278)

int		has_virtual_function,
		has_virtual_destructor;

if ( tag_begin )
	{
	if ( tag_kind > 1 && ! tag_nested && ! tag_local )
		{
		has_virtual_function = 0;
		has_virtual_destructor = 0;
		}
	}

if ( dcl_virtual )
	{
	if ( dcl_base == DESTRUCTOR_TYPE )
		has_virtual_destructor = 1;
	else if ( dcl_base != CONSTRUCTOR_TYPE )
		has_virtual_function = 1;
	}

if ( tag_end )
	{
	if ( tag_kind > 1 && ! tag_nested && ! tag_local )
		{
		if ( has_virtual_function && ! has_virtual_destructor )
			warn( 2007, "This class needs a virtual destructor." );
		has_virtual_function = 0;
		has_virtual_destructor = 0;
		}
	}

//////////////////////////////////////////////////////////////////////////
//	2. Use const variables or enumerated constants
//	   instead of preprocessor (#define) constants.

if ( pp_const )
	warn( 1001, "Do not use the preprocessor to define constants." );

//////////////////////////////////////////////////////////////////////////
//  3. A nested tag name should have an explicit scope.

if ( lex_invisible )
	warn( 1002, "Nested tag name %s should be scoped.", token() );

//////////////////////////////////////////////////////////////////////////
//  4. In C++ the const specifier implies internal linkage,
//     so either specify extern or initialize the constant.

if ( dcl_simple )
	if ( dcl_level_flags(0) & CONST_FLAG )
		if ( ! dcl_parameter && ! dcl_initializer && ! dcl_extern )
			warn( 2001, "Initializer or extern specifier needed." );

if ( dcl_level(0) == POINTER )
	if ( dcl_level_flags(0) & CONST_FLAG )
		if ( ! dcl_parameter && ! dcl_initializer && ! dcl_extern )
			warn( 2001, "Initializer or extern specifier needed." );

//////////////////////////////////////////////////////////////////////////
//  5. C++ variables should ALWAYS have an explicit type.

if ( dcl_no_specifier )
	{
	if ( dcl_function )
		warn( 2002, "%s: Explicit return type recommended.", dcl_name() );
	else
		warn( 2002, "%s: Explicit type specifier needed.", dcl_name() );
	}

//////////////////////////////////////////////////////////////////////////
//  6. Default function parameters are much safer than
//     variable argument lists, so use them!

if ( dcl_3dots )
	warn( 2003, "Use default parameters, not variable argument lists." );

//////////////////////////////////////////////////////////////////////////
//  7. Avoid gratuitous overloading of names in C++.

if ( dcl_hidden )
	warn( 2004, "Identifier %s is already in use.", dcl_name() );
