// Abraxas Rule File: Ellemtel C++ Rules 

// By Shuming Tan

// Copyright (C) ABRAXAS SOFTWARE, INC. 1994



#include < check.cch >


// ECR RULE  1: Include files in C++ always have the file name extension ".hh"

int hdr_name_len;

if (header_name() && pp_include !=4)
{
  hdr_name_len=strlen(header_name());
  if (header_name()[hdr_name_len-3]!='.')
    warn(1001,"Include file %s should have extension .hh",header_name());
  else 
    if (header_name()[hdr_name_len-2]!='h')
      warn(1001, "Include file %s should have extension .hh",header_name());
    else 
      if (header_name()[hdr_name_len-1]!='h')
        warn(1001, "Include file %s should have extension .hh",header_name());
}


// ECR RULE  2: Implementation files in C++ always have the file name extension .cc
int file_name_len;

if (mod_begin)
{
  file_name_len=strlen(file_name());
  if (file_name()[file_name_len-3]!='.')
    warn(1002,"Implementation file %s should have extension .cc",file_name());
  else
    if (file_name()[file_name_len-2]!='c')
      warn(1002,"Implementation file %s should have extension .cc",file_name());
    else 
      if (file_name()[file_name_len-1]!='c')
        warn(1002,"Implementation file %s should have extension .cc",file_name());
}


// ERC RULE  3: Inline definition files always have the file name extension ".icc".

int has_inline_def;

if (mod_begin) 
{
  has_inline_def=0;
}

if (dcl_inline) 
{
  has_inline_def=1;
}

if (has_inline_def)
{
  file_name_len=strlen(file_name());
  if (file_name_len<5)
    warn(1003,"Inline definition file %s should have extension .icc.",file_name());
  else 
    if (file_name()[file_name_len-4]!='.')
      warn(1003,"Inline definition file %s should have extension .icc.",file_name());
    else
      if (file_name()[file_name_len-3]!='i')
        warn(1003,"Inline definition file %s should have extension .icc.",file_name());
      else
        if (file_name()[file_name_len-2]!='c')
          warn(1003,"Inline definition file %s should have extension .icc.",file_name());
        else
          if (file_name()[file_name_len-1]!='c')
            warn(1003,"Inline definition file %s should have extension .icc.",file_name());
  has_inline_def=0;
}


// ERC RULE  4: Every file that contains source code must be documented with an
//              introductory comment that provides information on the file name 
//              and its contents.


// ERC RULE  5: All files must include copyright information.
//              Assume that the comment line inclding copy right info always at
//              the beginning of a file.

int copyright;

if (lin_end)
{
  if (strcmp(line(),"// Copyright XXX")==0) copyright=1;
  if (strcmp(line(),"/* Copyright XXX */")==0) copyright=1;
}

if (lin_preprocessor||lin_has_code)
{
  if (!copyright)
    warn(1005,"All files must include copyright information.");
  copyright=1;
}

if (mod_end) copyright=0;


// ERC RULE  6: All comments are to written in English
//              Oops, it is really hard to let computer judge 
//              if you are speaking English.


/*
// ERC RULE  7: Every include file must contain a mechanism that prevents
//              multiple includsions of the file
//              Maybe you have different naming convention to macros which serve
//              as header file wrapper. Suppose our is like this, for a include
//              file with name xxxx.yyy, the wrapping macro name should be
//              __XXXX__.


int lin_if_depth,		//  Holds latest value of pp_if_depth.
    wrapper_needed,		//  True when a header file is opened.
    bad_name,			//  True if the wrapper name is wrong.
    length, k, ch1, ch2;


if ( mod_begin )
{
  set_option('S',3);	//  Apply these rules to ALL headers.
}

if (pp_if_depth||pp_endif)
  lin_if_depth = pp_if_depth;

if (pp_include)
{
  wrapper_needed=1;
  if (lin_header&&lin_if_depth==0)   
    warn( 1007,"This #include should be wrapped in an #ifndef.");
}

if (lin_has_code)
  if (lin_header&&wrapper_needed)
  {
    warn(1007,"This header file needs a wrapper.");
    wrapper_needed=0;
  }

if (pp_macro)
  if (lin_header&&wrapper_needed)
  {
    wrapper_needed=0;
    bad_name=0;
    length=strlen(pp_name());
    if (pp_name()[0]!='_')
      bad_name = 1;
    else 
     if (pp_name()[1]!='_')
       bad_name=1;
     else 
       if (pp_name()[length-2]!='_')
         bad_name=1;
       else 
         if (pp_name()[length-1]!='_')
           bad_name=1;
         else
         {
           k=2;
           while (k<length-2)
           {
             ch1=pp_name()[k];
             ch2=file_name()[k-2];
             if (ch1 != toupper(ch2))
             {
               bad_name=1;
               break;
             }
             k++;
           }
  }

  if ( bad_name )
    warn( 1007, "%s is not the correct wrapper name for this file!", pp_name() );
  else 
    if ( lin_if_depth == 0 )
      warn( 1007, "This #define should be wrapped within an #ifndef." );
}

*/
// ERC RULE  8: When the following kinds of definitions are used (in 
//              implementation files or in other include files), they 
//              must be included as separate include files:
//
//              - classes that are used as base classes,
// 
//              - classes that are used as member variables,
//
//              - classes that appears as return types or as argument types
//                in function/member function prototypes.
//
//              - function prototypes for function/member functions used
//                in inline member functions that are defined in the file.



// ERC RULE  9: Definition of classes that are only accessed via pointers (*)
//              or references (&) shall not be included as include files.
//              This is the skeleton of the rule which will fit future CodeCheck
//              versions. The current version does not allow data structures like
//              array, struct, etc. Do not uncomment the code below, it is just 
//              psudo code.

/*
int cls_cnt;
int cls_idn_cnt;
int i;
int j;
int pointer_or_revference;

char class_in_header[100][64];
struct 
{
  char cls_name[64];
  int ptr_ref;
} cls_idn_tbl[100];

if (mod_begin)
{
  cls_cnt=0;
  cls_idn_cnt=0;
}

if (tag_begin&&lin_header==1)
{
  strcpy(class_in_header[cls_cnt],tag_name());
  cls_cnt++;
}

if (op_indirect||op_address)
{
  pointer_or_reference=1;
}

if (idn_variable&&lin_source)
{
  if (idn_base==CLASS_TYPE)
  {
    i=0;
    while (i<cls_idn_cnt)
    {
      if (strcmp(cls_idn_tbl[i].cls_name,idn_name())==0)
      {
        if (!pointer_or_reference)
        {
          cls_idn_tbl[i].ptr_ref=0;
        }
        break;
      }
      i++;
    }
    if (i==cls_idn_cnt&&pointer_or_reference)
    {
      strcpy(cls_idn_tbl[cls_idn_cnt].cls_name,idn_name());
      cls_idn_tbl[cls_idn_cnt].ptr_ref=1;
      cls_idn_cnt=++;
    }
  }
  pointer_or_reference=0;
}

if (mod_end)
{
  i=0;
  while (i<cls_idn_cnt)
  {
    if (cls_idn_tbl[i].ptr_ref)
    {
      j=0;
      while (j<cls_cnt)
      {
        if (strcmp(cls_idn_tbl[i].cls_name,class_in_header[j])==0)
        {
          warn(1009,"Definition of class %s that only accessed via * and & should not in include file.", class_in_header[j]);
          break;
        }
        j++;
      }
    }
    i++;
  }
}
  
*/


// ERC RULE 10: Never specify relative UNIX name in #include directives.

if (header_name())
{
  if (header_name()[0]=='/'||header_name()[0]=='.'||header_name()[0]=='~')
    warn(1010,"Never specifiy relative UNIX name in #include directives.");
}



// ERC RULE 11: Every implementation file is to include the relevant files
//              that contains
//
//              - declarations of types and functions used in the functions
//                that are implemented in the file.
//
//              - declarations of variables and member functions used in the
//                functions that are implemented in the file.



// ERC RULE 12: The identifier of every globally visible class, enumeration
//              type, type definition, function, constant, and variable in
//              a class library is to begin with a prefix that is unique for
//              the library.

if (tag_begin)
  if (tag_global)
    if (strstr(tag_name(),"MY")!=tag_name())
      warn(1012,"Global class name to begin with unique prefix MY.");

if (dcl_typedef)
  if (dcl_global)
    if (strstr(dcl_name(),"MY")!=dcl_name())
      warn(1012,"Global type definition to begin with unique prefix MY.");

if (dcl_variable)
  if (dcl_global)
    if (strstr(dcl_name(),"MY")!=dcl_name())
      warn(1012,"Global variable to begin with unique prefix MY.");

if (dcl_function)
  if (dcl_global)
    if (strstr(dcl_name(),"MY")!=dcl_name())
      warn(1012,"Global function to begin with unique prefix MY.");


// ERC RULE 13: The name of variables, constants, and functions are to begin 
//              with a lower case letter.

if (dcl_variable)
  if (dcl_first_upper)
    warn(1013,"The name of variable to begin with a lower case letter.");

if (dcl_function)
{
  if (dcl_base!=CONSTRUCTOR_TYPE&&dcl_base!=DESTRUCTOR_TYPE)
    if (dcl_first_upper)
      warn(1013,"The name of function to begin with a lower case letter.");
}


// ERC RULE 14: The name of abstract data types, structures, typedefs, and
//              enumerated types are to begin with uppercase letter.

if (tag_begin)
{
  if (!isupper(tag_name()[0]))
    warn(1014,"The name of abstract data types are to begin with uppercase letters.");
}

if (dcl_typedef)
{
  if (!isupper(dcl_name()[0]))
    warn(1014,"The name of typedefs are to begin with uppercase letters.");
}


// ERC RULE 15: In names which consist of more than one words are written
//              together and each word that follows the first is begin with
//              an uppercase letter.
//              Sorry, you have to do this yourself.


// ERC RULE 16: Do not use identifiers which begin with one or two underscores.

int leading_underscores;

if (dcl_global||dcl_local)
{
  leading_underscores=0;
  if (strlen(dcl_name())>2)
  {
    if (dcl_name()[0]=='_') leading_underscores=1;
    if (dcl_name()[1]=='_'&&leading_underscores==1) leading_underscores=2;
  }
  if (strlen(dcl_name())==2)
  {
    if (dcl_name()[0]=='_') leading_underscores=1;
  }
  if (leading_underscores)
  {
    warn(1016,"Do not use identifiers which begin with %d underscore(s).",leading_underscores);
  }
  leading_underscores=0;
}

if (idn_global||idn_local)
{
  leading_underscores=0;
  if (strlen(idn_name())>2)
  {
    if (idn_name()[0]=='_') leading_underscores=1;
    if (idn_name()[1]=='_'&&leading_underscores==1) leading_underscores=2;
  }
  if (strlen(idn_name())==2)
  {
    if (idn_name()[0]=='_') leading_underscores=1;
  }
  if (leading_underscores)
  {
    warn(1016,"Do not use identifiers which begin with %d underscore(s).",leading_underscores);
  }
  leading_underscores=0;
}


// ERC RULE 17: A name that begins with an uppercase letter is to appear directly
//              after it prefix.
//              Assume your prefix is "MY".
/*
int root_letter_1;
int root_letter_2;

if (prefix("MY"))
{
  root_letter_1=root()[0];
  root_letter_2=root()[1];
  if (root_letter_1=='_')
  {
    if (!islower(root_letter_2))
    {
      warn(1018,"Only use underscore to separate the name begin with lowercase from its prefix.");
    }
    else if (isupper(root_letter_2))
    {
      warn(1017,"The name begin with uppercase is to follow directly its prefix.");
    }
  }
  else
  {
    warn(1017,"Only name begin with uppercase is to follow directly it prefix.");
  }
}

*/
// ERC RULE 18: A name that begins with a lowercase letter is to be separated 
//              from its prefix using an underscore ('_').


// This rule is contained in the previous one.


// ERC RULE 19: A name is to be separated from its suffix using an underscore.
//              Assume your name suffix is XXX.

int root_len;
int root_last_letter;

if (suffix("XXX"))
{
  root_len=strlen(root());
  root_last_letter=root()[root_len-1];
  if (root_last_letter!='_')
    warn(1019,"A name is to separated from its suffix by an underscore.");
}



// ERC RULE 20: The public, protected, and private sections of a class are to 
//              be declared in that order (the public section is declared before
//              the protected section which is declared before the private section.)

int private_found;
int protected_found;
int in_class;

if (tag_begin&&tag_kind==CLASS_TAG) in_class=1;

if (keyword("public")&&in_class)
{
  if (private_found||protected_found)
    warn(1020,"Keep the declaration of sections in order PUBLIC, PROTECTED,PRIVATE.");
}

if (keyword("protected")&&in_class)
{
  if (private_found)
    warn(1020,"Keep the declaration of sections in order PUBLIC, PROTECTED,PRIVATE.");
  protected_found=1;
}

if (keyword("private"))
{
  private_found=1;
}

if (tag_mem_access)
{
  private_found=1;
}

if (tag_end)
{
  private_found=0;
  protected_found=0;
  in_class=0;
}


// ERC RULE 21: No member functions are to be defined within the class definition.

int in_class_body;

if (tag_begin&&tag_kind==CLASS_TAG)
{
  in_class_body=1;
}

if (fcn_begin)
  if (in_class_body)
    warn(1021,"No member functions are to be defined within the class definition.");

if (tag_end)
{
  in_class_body=0;
}


// ERC RULE 22: Never specify public or protected member data in a class.

int is_public_or_protected;

if ((keyword("public")||keyword("protected"))&&in_class)
{
  is_public_or_protected=1;
}

if (dcl_member)
{
  if (dcl_variable)
  {
    if (is_public_or_protected)
    {
      warn(1022, "Do not declare public or protected member data %s in class %s",dcl_name(),tag_name());
    }
  }
}

if (keyword("private")&&in_class)
{
  is_public_or_protected=0;
}


// ERC RULE 23: A member function does not affect the state of an object (its 
//              instance variables) is to be declared constant.


// ERC RULE 24: If the behavior of an object is dependent on data outside the object,
//              this data is not to be modified by const member functions.


// ERC RULE 25: A class which uses "new" to allocate instances managed by the class
//              must define a copy constuctor.


// ERC RULE 26: All classes which are used as base classes and which have virtual
//              functions must define a virtual destructor.

int has_virtual_destr;
int has_virtual_funcs;

if (dcl_function&&lin_within_class)
  if (dcl_virtual)
  {
    if (dcl_base==DESTRUCTOR_TYPE)
      has_virtual_destr=1;
    else 
      has_virtual_funcs=1;
  }

if (tag_end)
{
  if (has_virtual_funcs&&!has_virtual_destr)
    warn(1026,"This class need a virtual destructor.");
  has_virtual_funcs=0;
  has_virtual_destr=0;
}


// ERC RULE 27: A class which uses "new" to allocate instances managed by the class
//              must define an assignment operator.

// ERC RULE 28: An assignment operator which performs a destructive action must be
//              protected from performing this action on the object upon which it
//              is operating.


int in_op_func_assign;
int after_delete;

if (dcl_function)
{
  in_op_func_assign=0;
  {
    if (strcmp("operator=",dcl_name())==0)
    {
      in_op_func_assign=1;
    }
  }
}

if (op_delete&&in_op_func_assign)
{
  after_delete=1;
}

if (idn_variable)
{
  if (idn_member)
  {
    if (after_delete)
    {
      warn(1028,"Assigment operator has destructive action, X=X could have strange behavior.");
    }
  }
}

if (fcn_end)
{
  in_op_func_assign=0;
}

if (stm_end)
{
  after_delete=0;
}


      
// ERC RULE 29: A public member function must never return a non-const reference
//              or pointer to member data.

int is_pub_member_func;
int in_function;
int after_return;
int pointer_or_reference;
int ii;

if (dcl_function)
{
  if (dcl_definition)
  {
    if (dcl_member==3)
    {
      if (dcl_access==2)
      {
        is_pub_member_func=1;
      }
    }
  }
}



if (fcn_begin) 
{
  in_function=1;
  is_pub_member_func=0;
}

if (op_return&&pointer_or_reference) 
{
  after_return=1;
}

if ((op_address||op_indirect)&&in_function)
{
  pointer_or_reference=1;
}

if (idn_variable)
{
  pointer_or_reference=0;
  if (idn_member)
  {
    warn(0129,"Public member function never return a non-const pointer or reference to data member.");
  }
}

if (stm_end&&stm_kind==RETURN)
{
  after_return=0;
}

if (fcn_end)
{
  in_function=0;
  is_pub_member_func=0;
}



// ERC RULE 30: A public member function must never return a non-const reference 
//              or pointer to data outside an object, unless the object share the
//              data with other objects.



// ERC RULE 31: Do not use unspecified function arguments (ellipsis notation).

if (dcl_3dots)
  warn(1031,"Do not use unspecified function arguments(...).");


// ERC RULE 32: The name of formal arguments to functions are to be specified 
//              and are to be the same both in the function declaration and in 
//              the function definition.

// ERC RULE 33: Always specify the return type of function explicitly.

if (dcl_no_specifier)
  warn(1033,"Always specify the return type of function explicitly.");


// ERC RULE 34: A public function must never return a reference or pointer to
//              a local variable.

// ERC RULE 35: Do not use the preprocessor directive #define to obtain more efficient
//              code; instead, use inline functions.

if (pp_arg_count>0)
  warn(1035,"Do not use #define to obtain more efficient code use include functions.");


// ERC RULE 36: Constants are to be defined using const or enum; never use #define.

if (pp_const) 
  warn(1036,"Constant is to be defined using const or enum, not #define.");


// ERC RULE 37: Avoid the use of numeric values in code; use symbolic value instead.

if (lex_not_manifest)
  warn(1037,"Avoid the use of numeric values in code, use symbolic value instead.");


// ERC RULE 38: Variables to be declared with smallest possible scope.


// ERC RULE 39: Variables are to be declared with separate declaration statement.

if (dcl_count>1)
  warn(1039,"Variables are to be declared with separate declaration statement.");


// ERC RULE 40: Every variable that is to be given a value before it is used.


// ERC RULE 41: If possible, always use initialization instead of assignment.

if (dcl_variable)
  if (!dcl_parameter)
    if (!dcl_initializer)
      warn(1041,"Always use initialization instead of assignment if possible.");


// ERC RULE 42: Do not compare a pointer to NULL or assign NULL to a pointer; use 0
//              instead.

  
// ERC RULE 43: Never use explicit type conversions (casts).

if (op_cast)
  warn(1043,"Never use explicit type conversions (casts).");


// ERC RULE 44: Do not write code which depends on functions that use implicit
//              type conversion type conversions.


// ERC RULE 45: Never convert pointers to objects of a virtual base class.


// ERC RULE 46: Never convert a const to a non-const.

if (cnv_const_to_any)
  warn(1046,"Never convert a const to a non-const.");


// ERC RULE 47: The code followed a case label must always be terminated by a 
//              break statement.

if (stm_no_break)
  warn(1047,"Do not omit a break statement for previous case.");


// ERC RULE 48: A switch statement must always contain a default branch which handles 
//              unexpected cases.

if (stm_no_default)
  warn(1048,"A switch statement must have a default branch.");


// ERC RULE 49: Never use goto.

if (keyword("goto")) 
  warn(1049," Never use goto");


// ERC RULE 50: Do not use malloc, realloc or free.

if (idn_function)
{
  if (strcmp(idn_name(),"malloc")==0)
    warn(1050,"Do not use malloc.");
  if (strcmp(idn_name(),"realloc")==0)
    warn(1050,"Do not use realloc.");
  if (strcmp(idn_name(),"free")==0)
    warn(1050,"Do not use free.");
}


// ERC RULE 51: Always provide empty brackets "[]" for delete when deallocating arrays.

int after_delete_array;
int with_brackets;

if (op_delete)
{
  with_brackets=0;
  after_delete_array=1;
}

if (op_close_bracket&&after_delete_array)
{
  with_brackets=1;
}

if (idn_variable)
{
  if (idn_levels==1&&(idn_level(0)==ARRAY||idn_level(0)==POINTER))
  {
    if (!with_brackets&&after_delete_array)
      warn(1051,"Always provide empty bracket for delete when deallocating arrays.");
  }
  after_delete_array=0;
}


