/*	fcndebug.cc

	Copyright (c) 1992-93 by Abraxas Software. All rights reserved.
==============================================================================
Purpose:	Checks for presence of FCN_DEBUG macro call at every
			function entry and exit point.
Author:		Loren Cobb.
Revision:	5 September 1992.
Format:		Monospaced font with 4 spaces/tab.
Thanks to:	Terry McNulty, AT&T, for this idea.
==============================================================================

Abstract:

	For debugging purposes it is sometimes necessary to place a call to some
	function (often just printf) at the beginning and end of every function.
	
	These rules check that a macro function named FCN_DEBUG has been found
	at every entry and exit point of every function.

	The exit point of a function is either a return statement or the end of
	the function (if it has no return).
	
	The macro function FCN_DEBUG can be as elaborate as desired, or as simple
	as this:
	
		#ifdef DEBUG
		#define FCN_DEBUG(message)   fprintf( stderr, message );
		#else
		#define FCN_DEBUG(message)
		#endif
	
	The rule that "a return must be enclosed in curly braces if it is the
	body of an if statement" is necessary to ensure that the macro FCN_DEBUG
	really will be called at the exit point. We are trying to prevent this:
	
		FCN_DEBUG( "About to exit." );
		if ( foobar )
			return;
		//  more executable code here.
	

==============================================================================
*/

#include <check.cch>

int debug_found;   //  1 if FCN_DEBUG macro was found on this line.

int	need_debug;    //  2 if FCN_DEBUG macro was found on this line.   
                   //  1 if FCN_DEBUG macro was found on the previous line.

int return_found;  //  2 if a return was found on this line.   
                   //  1 if a return was found on the previous line.

if ( op_macro_call )
	if ( strcmp(op_macro(),"FCN_DEBUG") == 0 )
		debug_found = 1;

if ( fcn_begin )		//  Open curly brace found.
	need_debug = 2;

if ( lin_end )
	{
	if ( need_debug )
		{
		if ( need_debug == 2 )    //  Ignore the line containing
			--need_debug;         //  the open curly brace.
		else
			{
			if ( ! debug_found )
				warn( 1, "FCN_DEBUG needed at function entry." );
			need_debug = 0;
			}
		}
	if ( return_found )
		--return_found;
	debug_found = 0;
	}

if ( stm_kind == RETURN )
	{
	return_found = 2;
	if ( ! debug_found )
		warn( 2, "FCN_DEBUG needed before this return." );
	if ( (stm_container == IF) || (stm_container == ELSE) )
		warn( 3, "Enclose this return statement in curly braces." );
	}

if ( fcn_end )
	{
	if ( (! debug_found) && (! return_found) )
		warn( 4, "FCN_DEBUG needed at end of function." );
	}
