/*	guide.cc
	Copyright (c) 1994 by Abraxas Software. All rights reserved.
==============================================================================
	Purpose:	Applies C guidelines
	Author:		Shuming Tan.
	Revision:	8 August 1994.
	Format:		Use a monospaced font.
==============================================================================
*/

#include <check.cch>

int v_flag;
int t;


/*
************************************************************************************
* 1. This rule is to check the use of keyword "void". To help increase the                                              *
*    portability. Use VOID instead of void, VOIDPTR instead of void *,                                                 *
*    VOIDPTR * instead of void **. Also use CONST instead of const.                                                  *
************************************************************************************
*/

/* When keyword "const" is found and it is not derived from a macro 
   definition, warn the use of "const". */
if (keyword("const")&&!lex_macro_token)
  warn(1001,"Do not use const, use CONST.");

/* When void and void* are detected, warn the use of them */
if (keyword("void")) v_flag=1;

if ((dcl_local||dcl_global||dcl_extern)&&v_flag&&!(dcl_typedef||lex_macro_token))
{
  switch (dcl_levels)
  {
    case 0:
      warn(1002,"Do not use void, use VOID");
      break;
    case 1:
      if (dcl_level(0)==POINTER)
        warn(1002,"Do not use void*, use VOIDPTR");
      else
        warn(1002,"Do not use void, use VOID.");
      break;
    default:
      if (dcl_level(1)==POINTER)
        warn(1002,"Do not use void*, use VOIDPTR.");
      else
        warn(1002,"Do not use void, use VOID.");
  }
  v_flag=0;
}


/*
************************************************************************************
* 2. This rule finds ANSI keywords that are used outside of their normal role.                                                                                                                                                         *
************************************************************************************
*/

/* the case that errno declared as a member of structure */
if (dcl_member==2)
  if (strcmp(dcl_name(),"errno")==0)
    warn(2001,"Do not use \"errno\" as a structure member.");

/* the case that a member of structure declared with void type */
if (dcl_member==2)
  if (dcl_base==VOID_TYPE)
    warn(2002,"Do not define structure member as void type.");

/* the case that a member of structure declared with extern storage flag */
if (dcl_extern)
  if (dcl_member==2)
    warn(2003,"Do not define structure member with extern flag.");

/* the case that a member of structure define as a constant */
if (dcl_member==2)
  if (dcl_level_flags(dcl_levels)&CONST_FLAG)
    warn(2004,"Do not define structure member with const flag.");


/*
************************************************************************************
* 3. This rule is about the name convention. All user defined function names can                                  *
*    not begin with _ (underscore), is, to, mem, str, f. All user defined macros                                        *
*    can not begin with LC_ and E. All user defined structures can not end with _t.                                *                        
************************************************************************************
*/

/*
 * when a function declared, check if it begins with prefixes
 * "_","is","to","str","mem" or "f". 
*/
if (dcl_function)
{
  if (strstr(dcl_name(),"_")==dcl_name())
    warn(3001,"No user defined function can begin with \"_\".");
  if (strstr(dcl_name(),"is")==dcl_name())
    warn(3002,"No user defined function can begin with \"is\".");
  if (strstr(dcl_name(),"to")==dcl_name())
    warn(3003,"No user defined function can begin with \"to\".");
  if (strstr(dcl_name(),"mem")==dcl_name())
    warn(3004,"No user defined function can begin with \"mem\".");
  if (strstr(dcl_name(),"str")==dcl_name())
    warn(3005,"No user defined function can begin with \"str\".");
  if (strstr(dcl_name(),"f")==dcl_name())
    warn(3006,"No user defined function can begin with \"f\".");
}

/*
 * when a structure or a typedef from a structure with name ends with "_t"
*/

if (dcl_base==STRUCT_TYPE)
  if (dcl_tag_def)
    if (strstr(dcl_base_name(),"_t")==dcl_base_name()+strlen(dcl_base_name())-2)
      warn(3007,"No user defined structure can end with \"_t\".");

if (dcl_typedef)
  if (dcl_base==STRUCT_TYPE)
    if (strstr(dcl_name(),"_t")==dcl_name()+strlen(dcl_name())-2)
      warn(3007,"No user defined structure can end with \"_t\".");

/*
 * when a macro is defined, check if it begins with prefixes "E" or "LC"
*/

if (pp_macro)
{
  if (strstr(pp_name(),"LC_")==pp_name())
    warn(3008,"No user defined macro can begin with \"LC_\".");
  if (strstr(pp_name(),"E")==pp_name())
    warn(3008,"No user defined macro can begin with \"E\".");
}


/*
************************************************************************************
* 4. This rule is to check if function malloc, calloc and free are redefined or                                            *
*    defined as macro by user.                                                                                                                     *
************************************************************************************
*/

/* notify when find that function "malloc","calloc" or "free" are
 * redefined by user, or these names are defined as macros' name
*/
if (dcl_function)
  if (strcmp(dcl_name(),"malloc")==0)
    warn(4001,"Do not redefine function \"malloc\".");

if (dcl_function)
  if (strcmp(dcl_name(),"calloc")==0)
    warn(4003,"Do nor redefine function \"calloc\".");

if (dcl_function)
  if (strcmp(dcl_name(),"free")==0)
    warn(4005,"Do not redefine function \"free\".");

if (pp_macro)
  if (strcmp(pp_name(),"malloc")==0)
    warn(4002,"Do not define \"malloc\" as macro.");

if (pp_macro)
  if (strcmp(pp_name(),"calloc")==0)
    warn(4004,"Do not define \"calloc\" as macro.");

if (pp_macro)
  if (strcmp(pp_name(),"free")==0)
    warn(4006,"Do not define \"free\" as macro.");
/*
************************************************************************************
* 5. All structures should have a tag name to define structure, and may also have                                            *
*    a separate user-defined typedef name if desired.                                                                                          *
************************************************************************************
*/

/*
 * when a structure defined, it should have a name to define structure
*/
if (tag_end) t=1;

if (dcl_base==STRUCT_TYPE&&t)
  if (!dcl_typedef)
    if (!dcl_initializer)
      if (strcmp("}",prev_token()))
      {
        warn(5001,"Do not declare variable here, do it later.");
        t=0;
      }
/* suggest use a typedef first, then use the typedef name to declare the variables */
if (dcl_base==STRUCT_TYPE)
  if (!dcl_tag_def&&!dcl_parameter&&!dcl_typedef)
    warn(5001,"Do not use struct in declaration directly.");

/*
************************************************************************************
* 6. Do not use typedef in function prototypes.                                                                                         *
************************************************************************************
*/

/* do not use typedef as the type of parameter. However, right now there
 * is no way to find out what kind of type this typedef derives from.
*/
if (dcl_parameter)
  if (dcl_base==DEFINED_TYPE)
   warn(1000,"Do not use typedef in function prototype.");

/*
************************************************************************************
* 7. This rule can be used to check if the header file containing the library                                             *
*    function just used has been included  into the module.                                                                       *
************************************************************************************
*/

/* check the used functions and variables defined in header files, if the 
 * corresponding header file which contains the found function has not been
 * included into, give a warning message. Not all functions and header files
 * are included, most of them can be done in the same way below.
*/
if (op_call)
{
    if (strstr(op_function(),"time")==op_function()+strlen(op_function())-4)
      if (!included("time.h"))
        warn(7001,"Header file <time.h> should be included.");
}

if (op_call)
{
  if (!included("ctype.h"))
  {
    if (strstr(op_function(),"is")==op_function())
      warn(7002,"header file <ctype.h> should be included.");
    if (strstr(op_function(),"to")==op_function())
      warn(7002,"Header file <ctype.h> should be included");
  }
}

if (op_call)
{
  if (!included("string.h"))
  {
    if (strstr(op_function(),"str")==op_function())
      warn(7003,"Header file <string.h> should be included.");
    if (strstr(op_function(),"mem")==op_function())
      warn(7003,"Header file <string.h> should be included.");
  }
}

if (dcl_extern)
  if (strcmp(dcl_name(),"errno")==0)
    if (!included("errno.h"))
      warn(7004,"Header file <errno.h> should be included.");


if (idn_variable)
  if (idn_not_declared)
  {
    if (strstr(prev_token(),"E")==prev_token())
      if (!included("errno.h"))
        warn(7004,"Header file <errno.h> should be included.");

    if (strstr(prev_token(),"LC_")==prev_token())
      if (!included("locale.h"))
        warn(7005,"Header file <locale.h> should be included.");
  }


if (op_call)
{
  if (!included("stdlib.h"))
  {
    if (strcmp(op_function(),"malloc")==0)
      warn(7006,"Header file <stdlib.h> should be included.");
    if (strcmp(op_function(),"calloc")==0)
      warn(7006,"Header file <stdlib.h> should be included.");
    if (strcmp(op_function(),"free")==0)
      warn(7006,"Header file <stdlib.h> should be included.");
    if (strstr(op_function(),"rand")==op_function()+strlen(op_function())-4)
      warn(7006,"Header file <stdlib.h> should be included.");
  }
}

if (op_call)
  if (!included("assert.h"))
    if (strcmp(op_function(),"assert")==0)
       warn(7007,"Header file <assert.h> should be included.");

/* once a hearder file is not found, warn to put it in the 
 * predefined #ifndes at the top of all files.
*/
if (pp_not_found)
  warn(7008,"Header file \"%s\" not found, put it in #ifudef at file top.",token());

/*
************************************************************************************
* 8. When function calling sequence changed or new functions added, remember to                              *
*    update the appropriate prototype header files.                                                                                    *
************************************************************************************
*/
/*
Currently, CodeCheck does not have the capacity to reach this specific goal.
To reach this goal, we need to have more complicated data structures, such as
struct, array and file, which are not available in current version. Now only 
simple data type (integer, float and char, for string, only constant string)
are allowed to be used in CodeCheck. With those complicated data type becoming
available, the implement the rule which monitors the change of calling sequence
and addition of new functions will be staight. Simply check the current function
calling sequence and function name table agianst the record file. Once the 
descrepencies found, take the action desired.
*/
