/* ================================================================

                       INDENTATION

          Copyright (c) 1990-1993 by Abraxas Software.

These rules check for correct indentation and placement of braces.
NOT checked are the following:
	- comment lines,
	- preprocessor lines,
	- lines outside of function definitions,
	- lines that continue expressions from the previous line,
	- lines that begin with a label.

LAST REVISED:  5/17/94 by Loren Cobb.

MESSAGES:

Warning 1001:  Put this open brace on its own line.
Warning 1002:  Put this close brace on its own line.
Warning 1003:  Indentation needs a minor adjustment.
Warning 1004:  Indentation is way off base.

NOTE:  If you want programmers to indent their braces, run
this rule file with command-line option -B, or use this rule:

		if ( mod_begin )
			{
			set_option( 'B', 1 );
			spaces_per_tab = option( 'X' );
			}

----------------------------------------------------------------- */

int		difference,		//  Measures indentation error in tabs.
		spaces_per_tab;	//	Desired spaces per tab (option -Xn)


/*	--------------------------------------------------------------
	Initialization:
	If the -X option is found on the command line with an integer
	value, then use this value for spaces_per_tab (default = 4).
*/

if ( mod_begin )
	{
	spaces_per_tab = option( 'X' );
	if ( spaces_per_tab == 0 )
		spaces_per_tab = 4;
	}

/*	--------------------------------------------------------------
	Rule 1001:
	Every compound statement in a function must begin
	with an open brace alone on a line, except after
	the "do" keyword.
*/

if ( stm_cp_begin )
	if ( lin_tokens > 1 )
		if ( strcmp(prev_token(),"do") )
			warn( 1001, "Put this open brace on its own line." );

/*	--------------------------------------------------------------
	Rule 1002:
	Every compound statement in a function must end
	with an open brace alone on a line.
*/

if ( stm_is_comp )
	if ( lin_tokens > 1 )
		warn( 1002, "Put this close brace on its own line." );

/*	--------------------------------------------------------------
	Rule 1003:
	Every line containing executable code must be properly indented,
	unless it is a comment, a continuation of an expression from the
	previous line, or a line that begins with a label.
*/

if ( lin_within_function )
	if ( ! (lin_is_comment || lin_preprocessor || lin_continuation || lin_has_label) )
		{
		if ( lin_indent_space )
			{
			difference = lin_nest_level*spaces_per_tab - lin_indent_space;
			switch ( difference )
				{
				case -2:
					warn( 1003, "Indent 2 fewer spaces." );
					break;
				case -1:
					warn( 1003, "Indent 1 fewer space." );
					break;
				case 0:		//	Indentation is correct.
					break;
				case 1:
					warn( 1003, "Indent 1 more space." );
					break;
				case 2:
					warn( 1003, "Indent 2 more spaces." );
					break;
				default:
					warn( 1004, "Indent %d spaces.", difference );
					break;
				}
			}
		else
			{
			difference = lin_nest_level - lin_indent_tab;
			switch ( difference )
				{
				case -2:
					warn( 1003, "Indent 2 fewer tabs." );
					break;
				case -1:
					warn( 1003, "Indent 1 fewer tab." );
					break;
				case 0:		//	Indentation is correct.
					break;
				case 1:
					warn( 1003, "Indent 1 more tab." );
					break;
				case 2:
					warn( 1003, "Indent 2 more tabs." );
					break;
				default:
					warn( 1004, "Indent %d tabs.", difference );
					break;
				}
			}
		}


