//   This CodeCheck rule file checks C sources for compatibility
//   with the limits and anomalies of the Intel iC-386 V4.2 compiler.

//   Use these rules when you port code to any Intel C compiler.

//   Copyright (c) 1990-93 by Abraxas Software, Inc.


#include <check.cch>

int   mod_extern_refs;   //  Counts number of external references per module.

if ( mod_begin )
	mod_extern_refs = 0;

if ( dcl_global )
	if ( ! dcl_static )
		++mod_extern_refs;

if ( mod_end )
	{
	if ( mod_extern_refs > 511 )
		warn( 1, "External references = %d (iC-386 maximum is 511 per module).", mod_extern_refs );

	if ( mod_functions > 1022 )
		warn( 2, "Function definitions = %d (iC-386 maximum is 1022 per module).", mod_functions );
	}

if ( lex_nonstandard == '$' )
	warn( 3, "The iC-386 \"extend\" control will affect this identifier." );

if ( lex_wide )
	warn( 4, "iC-386 V4.2 will truncate wide character constants." );

if ( pp_stack )
	warn( 5, "Macro redefinition is not allowed by iC-386." );

if ( pp_if_depth > 16 )
	warn( 6, "Conditional compilation depth exceeds 16." );

if ( pp_arg_count > 31 )
	warn( 7, "Too many macro parameters (iC-386 maximum is 31)." );

if ( dcl_level(0) == ARRAY )
	if ( (dcl_base == CHAR_TYPE) || (dcl_base == UCHAR_TYPE) )
		if ( dcl_initializer )
			warn( 8, "String initializers are NOT null-terminated in iC-386." );

if ( dcl_parm_count > 31 )
	warn( 9, "Too many function parameters (iC-386 maximum is 31)." );

if ( dcl_no_specifier )
	{
	if ( dcl_function )
		warn( 10, "This function needs an explicit return type." );
	else
		warn( 11, "This declaration must have an explicit type." );
	}

if ( dcl_need_3dots )
	warn( 12, "This argument list must end with 3 dots (...)." );
	
if ( identifier("fgetpos") )
	warn( 13, "The iC-386 V4.2 libraries do not contain fgetpos." );

if ( identifier("fsetpos") )
	warn( 14, "The iC-386 V4.2 libraries do not contain fsetpos." );

if ( identifier("fpos_t") )
	warn( 15, "The iC-386 V4.2 headers do not define the type fpos_t." );

if ( stm_depth > 32 )
	warn( 16, "Statements nested too deeply (iC-386 maximum is 32)." );

if ( keyword("huge") || keyword("pascal") || keyword("fortran") )
	warn( 17, "This keyword is not defined in iC-386 C." );
