/*
 *  Copyright (c) 1988-93 by Abraxas Software.
 *
 *	This is the start of a collection of CodeCheck rules
 *	for testing for portability to VAX computers, running
 *  under either VMS or ULTRIX.
 *
 *  Reference: Ultrix-32 Guide to VAX C, 1988 edition.
 */


#include <check.cch>

/**********************************************************************
ANSI suffixes U and F are not supported by VAX C.
************************************************************************/
if ( lex_unsigned )
	warn( 4001, "Do not use the suffix U." );
if ( lex_float )
	warn( 4002, "Do not use the suffix F." );


/**********************************************************************
"No bitfield can be longer than 32 bits in VAX C."
"Bitfields must be of data types unsigned or int."
-- reference: Ultrix-32 Guide to VAX C, Section 7.12.5
************************************************************************/
if ( dcl_bitfield )
	{
	if ( dcl_bitfield_size > 32 )
		warn( 4004, "Bitfield exceeds 32 bits - too wide." );
	if ( dcl_base != INT_TYPE )
		warn( 4005, "Bitfields must be int or unsigned." );
	}


/**********************************************************************
"Vax C will not initialize local variables when control passes into a
 block by means of a goto, or if the block is the body of a switch."
-- reference: Ultrix-32 Guide to VAX C, Section 8.3.1
************************************************************************/
if ( stm_goto )
	warn( 4006, "Local variable initialization will not occur here." );


/**********************************************************************
VAX C has 12 predefined identifiers for use with the _align modifier.
These identifiers may not be used for any other purpose.
-- reference: Ultrix-32 Guide to VAX C, Section 8.8.3
************************************************************************/
if ( identifier("BYTE") || identifier("byte") )
    warn( 4007, "BYTE and byte may not be used as VAX C identifiers." );
if ( identifier("WORD") || identifier("word") )
    warn( 4007, "WORD and word may not be used as VAX C identifiers." );
if ( identifier("LONGWORD") || identifier("longword") )
    warn( 4007, "LONGWORD and longword may not be used as VAX C identifiers." );
if ( identifier("QUADWORD") || identifier("quadword") )
    warn( 4007, "QUADWORD and quadword may not be used as VAX C identifiers." );
if ( identifier("OCTAWORD") || identifier("octaword") )
    warn( 4007, "OCTAWORD and octaword may not be used as VAX C identifiers." );
if ( identifier("PAGE") || identifier("page") )
    warn( 4007, "PAGE and page may not be used as VAX C identifiers." );


/**********************************************************************
VAX C does not allow a variable to be initialized when the extern specifer
is used (as in: extern short xyz = 123;).
************************************************************************/
if ( dcl_extern )
	if ( dcl_initializer )
		warn( 4008, "Initialization of an extern variable." );


/**********************************************************************
The VAX C preprocessor will go into recursive death when it tries to
expand a recursive macro.
************************************************************************/
if ( pp_recursive )
	warn( 4009, "This macro will cause recursive death." );


if ( pp_error )
	warn( 4010, "The VAX C preprocessor does not support #error directives." );
