/*
***************************************************************************
*                                                                         *
* FILE NAME:	dcl.cc                                                    *
* COPYRIGHT (C) 1994 BY ABRAXAS SOFTWARE. ALL RIGHTS RESERVED.            *
* AUTHOR:	Shuming Tan                     July 21, 1994             *
* PURPOSE:	This a CodeCheck rule file used to test the predefined    *
*               variables in dcl_ group.                                  *
*                                                                         *
***************************************************************************
*/
#include <check.cch>

int i;
/* test the variables about the lexical form of identifiers */
/*#########################################################################################*/
if (dcl_all_upper)
  warn(2001,"A declarator name all uppercase");

if (dcl_any_upper)
  warn(2002,"A declarator name with uppercase letters");

if (dcl_first_upper)
  warn(2003,"A declarator with beginning uppercase letter");

if (dcl_Hungarian)
  warn(2004, "A declarator using Hungarian convention");

if (dcl_extern_ambig)
  warn(2005,"An external identifier's first %d chars match someone else",dcl_extern_ambig);

if (dcl_ambig)
  warn(2006,"A declarator's first %d chars match another identifier",dcl_ambig);

if (dcl_underscore)
  warn(2007, "%d leading underscore in declarator name",dcl_underscore);

if (dcl_from_macro)
  warn(2008, "A declarator name expanded from macro");
/*#########################################################################################*/

/* test the variables about the scope of the declaration */
/*#########################################################################################*/
if (dcl_global)
  warn(2009, "A declarator with file scope");

if (dcl_local)
  warn(2010, "A local identifier declared");

if (dcl_conflict)
  warn(2011,"An identifier declared somewhere else");

if (dcl_enum_hidden)
  warn(2012,"An enumerated constant hidden by a declarator name");

if (dcl_hidden)
  warn(2013, "A local variable hide another variable");

if (dcl_parm_hidden)
  warn(2014, "A function parameter hidden by a local variable");

if (dcl_label_overload)
  warn(2015, "An declarator name matches a label name");
/*#########################################################################################*/

/* test variables about declaration type */
/*#########################################################################################*/
if (dcl_count)
  warn(3001, "There are %d declarators in this declaration",dcl_count);

if (dcl_simple)
  warn(2016, "A simple variable declared");

if (dcl_aggr)
  warn(2017,"An aggregate type declared");

if (dcl_enum)
  warn(2018,"An enumerated constant found");

if (dcl_variable)
{
  warn(2019, "A variable declared");
  warn(2020, "Identifier has %d characters", dcl_ident_length);
}

if (dcl_unsigned)
  warn(2021, "A declarator is unsigned");

if (dcl_signed)
  warn(3021, "A delcarator is singed");

if (dcl_long_float)
  warn(2022, "A variable declared LONG FLOAT");

if (dcl_static)
  warn(2023, "A static declarator");

if (dcl_definition)
  warn(2024,"A declaration a definition, not a reference");

if (dcl_extern)
  warn(2025,"An EXTERN declared explicitly");

if (dcl_tag_def)
  warn(2026, "A tag is part of a type specifier");

if (dcl_typedef)
  warn(2027, "A typedef name declared");

if (dcl_function_ptr)
  warn(2028, "A pointer to a function declared");

if (dcl_function)
{
  warn(2029, "A function or function tyepdef name declared");
  warn(2030, "The function has %d parameters",dcl_parm_count);
}

if (dcl_member)
{
  switch (dcl_member)
  {
    case 1: 
      warn(2031, "A union member identifier declared");
      break;
    case 2:
      warn(2031, "A struct member identifier declared");
      break;
    case 3:
      warn(2031, "A class member identifier declared");
      break;
    default:
      warn(2031, "A unknown member identifier declared");
  }      
}
/*#########################################################################################*/

/* test variables about bitfield */
/*#########################################################################################*/
if (dcl_bitfield)
{
  warn(2032, "A bitfield declared");
  warn(2033, "The bitfield has %d bits",dcl_bitfield_size);
}

if (dcl_bitfield_anon) 
  warn(2034, "A bitfield with no name");

if (dcl_bitfield_arith)
  warn(2035, "A bitfield width requires arithmetic calculation");

if (dcl_union_bits)
  warn(2036, "A bitfield declared as member of a union");
/*#########################################################################################*/


/* test variables about irregular declarations */
/*#########################################################################################*/
if (dcl_abstract)
  warn(2037,"An abstract declarator encountered");

if (dcl_empty)
  warn(2038,"An empty declaration(no declarator)");

if (dcl_no_specifier)
  warn(2039, "A declarator without type specifier");

if (dcl_storage_first)
  warn(2040, "A storage specifier preceded by a type specifier");

if (dcl_typedef_dup)
  warn(2041, "A duplicated typedef name declared");

if (dcl_cv_modifier)
  warn(2042, "CONST or VOLATILE used as non_ANSI modifier");
/*#########################################################################################*/


/* test variables about initializer */
/*#########################################################################################*/
if (dcl_init_arith)
  warn(2043, "An initializer using arithmetic");

if (dcl_initializer)
  warn(2044, "An initializer found");

if (dcl_auto_init)
  warn(2045, "An auto variable initialized");

if (dcl_union_init)
  warn(2046, "A union has an initializer");
/*#########################################################################################*/


/* test variables about C++ features */
/*#########################################################################################*/

if (dcl_inline)
  warn(2047,"A C++ inline function");

if (dcl_virtual)
  warn(2048, "A virtual function declared");

if (dcl_friend) 
  warn(2049,"A C++ friend declared");

if (dcl_pure)
  warn(2050, "A C++ pure function declared");

if (dcl_template)
  warn(2051, "The template has %d parameters",dcl_template);
/*#########################################################################################*/

/* test the variables about the irregular function definitions */
/*#########################################################################################*/
if (dcl_no_prototype)
  warn(2052, "A function definition not prototyped");

if (dcl_not_declared)
  warn(2053, "An old-style function parameter not declared");

if (dcl_oldstyle)
  warn(2054, "An old-style(unprototyped) function declared");

if (dcl_need_3dots)
  warn(2055, "A parameter list concludes with a comma");

if (dcl_3dots) 
  warn(2056,"An ellipsis (...) found in declaration");

if (dcl_type_before)
  warn(2057, "Return type of a function definition on the previous line");
/*#########################################################################################*/

/* test variables about array declarations */
/*#########################################################################################*/
if (dcl_levels)
{
  if (dcl_level(0)==ARRAY)
  {
    warn(2058, "Array dimension is %d",dcl_array_dim(0));
    warn(2059, "Array size is %d",dcl_array_size);
    if (dcl_zero_array) 
      warn(2060, "Array has zero size");
  }
}
/*#########################################################################################*/




if (dcl_base)
{
  switch (dcl_base)
  {
    case VOID_TYPE:
      warn(2061, "Base type is VOID");
      break;
    case CHAR_TYPE:
      warn(2061, "Base type is CHAR");
      break;
    case SHORT_TYPE:
      warn(2051, "Base type is SHORT");
      break;
    case INT_TYPE:
      warn(2061, "Base type is INT");
      break;
    case LONG_TYPE:
      warn(2061, "Base type is LONG");
      break;
    case EXTRA_INT_TYPE:
      warn(2061, "Base type is EXTRA INT");
      break;
    case UCHAR_TYPE:
      warn(2061, "Base type is UNSIGNED CHAR");
      break;
    case USHORT_TYPE:
      warn(2061, "Base type is UNSIGNED SHORT");
      break;
    case UINT_TYPE:
      warn(2061, "Base type is UNSIGNED INT");
      break;
    case ULONG_TYPE:
      warn(2051, "Base type is UNSIGNED LONG");
      break;
    case EXTRA_UINT_TYPE:
      warn(2061, "Base type is EXTRA UNSIGNED INT");
      break;
    case FLOAT_TYPE:
      warn(2061, "Base type is FLOAT");
      break;
    case DOUBLE_TYPE:
      warn(2061, "Base type is DOUBLE");
      break;
    case LONG_DOUBLE_TYPE:
      warn(2061, "base type is LONG DOUBLE");
      break;
    case EXTRA_FLOAT_TYPE:
      warn(2061, "Base type is EXTRA FLOAT");
      break;
    case ENUM_TYPE:
      warn(2061, "Base type is ENUM");
      break;
    case UNION_TYPE:
      warn(2051, "Base type is UNION");
      break;
    case STRUCT_TYPE:
      warn(2061, "Base type is STRUCT");
      break;
    case CLASS_TYPE:
      warn(2061, "Base type is CLASS");
      break;
    case DEFINED_TYPE:
      warn(2061, "Base type is DEFINED");
      break;
    case EXTRA_PTR_TYPE:
      warn(2061, "Base type is EXTRA POINTER");
      break;
    case CONSTRUCTOR_TYPE:
      warn(2061, "Base type is CONSTRUCTOR");
      break;
    case DESTRUCTOR_TYPE:
      warn(2061, "Base type is DESTRUCTOR");
      break;
    default:
      warn(2061, "Base type is UNKNOWN");
  }
}

if (dcl_storage_flags)
{
  if (dcl_storage_flags&EXTERN_SC) 
    warn(2071, "It has a EXTERN storage flag");
  if (dcl_storage_flags&STATIC_SC)
    warn(2072, "It has a STATIC storage flag");
  if (dcl_storage_flags&TYPEDEF_SC)
    warn(2073, "It has a TYPEDEF storage flag");
  if (dcl_storage_flags&AUTO_SC)
    warn(2074, "It has an AUTO storage flag");
  if (dcl_storage_flags&REGISTER_SC)
    warn(2075, "It has a REGISTER storage flag");
  if (dcl_storage_flags&GLOBAL_SC)
    warn(2076, "It has a GLOBAL a storage flag");
}


if (dcl_variable||dcl_function)
{
  i=0;
  while (i<=dcl_levels)
  {
    switch (dcl_level(i))
    {
      case SIMPLE:
        warn(2085, "Level %d is SIMPLE",i);
        break;
      case FUNCTION:
        warn(2085, "Level %d is FUNCTION",i);
        break;
      case REFERENCE:
        warn(2085, "Level %d is REFERENCE",i);
        break;
      case POINTER:
        warn(2085, "Level %d is POINTER",i);
        break;
      case ARRAY:
        warn(2085, "Level %d is ARRAY",i);
        break;
      default:
        warn(2085, "Level %d is UNKNOWN",i);
    }
    i++;
  }
} 

   

